/**
 * 
 */
package de.mcs.tools.sps.emulator;

import java.util.List;

import de.mcs.tools.sps.emulator.exceptions.WrongProgramSizeException;

/**
 * @author w.klaas
 *
 */
public interface SPSEmulator {

  /**
   * returning all commands supported by this emulator
   * 
   * @return lsit of all supported commands
   */
  List<SPSCommand> getCommands();

  /**
   * @return the size of the internal program area
   */
  int getProgramSize();

  /**
   * loading a program into the emulator
   * 
   * @param program
   *          to load
   * @throws WrongProgramSizeException
   */
  void loadProgram(byte[] program) throws WrongProgramSizeException;

  /**
   * starting the program with or without debug mode
   * 
   * @param debug
   *          if debug = <code>true</code> than the program will stop on every step, otherwise only the callbacks will
   *          be called
   */
  void startProgram(boolean debug);

  /**
   * adding a callback for the outputs of the emulator
   * 
   * @param callback
   */
  void addCallback(TOutputCallback callback);

  /**
   * do the next step of the program
   */
  void nextStep();

  /**
   * sps emulator is active
   * 
   * @return
   */
  boolean isActive();

  /**
   * stop the actual process
   */
  void stop();

  /**
   * setting the input for the emulator
   * 
   * @param input
   */
  void setEmulatorInput(EmulatorInput input);

  EmulatorInput getEmulatorInput();

  /**
   * 
   * @return the actual emulator internals
   */
  EmulatorInternals getEmulatorInternals();

  EmulatorOutput getEmulatorOutput();

  SPSCommandData getCommandData(byte command);
}
