package de.mcs.tools.sps.mnemonic;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import de.mcs.tools.sps.exceptions.IllegalArgument;
import de.mcs.tools.sps.exceptions.SyntaxError;

class TestLOOPC {

  private static final int BYTE_VALUE = 0xA0;
  private final String MNEMONIC = "LOOPC";

  @Test
  void testMnemonic() throws SyntaxError {
    LOOPC mnemonic = new LOOPC(MNEMONIC + " 0x00");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE, mnemonic.getByte());
    assertTrue(mnemonic.hasArgument());
    assertTrue(mnemonic.isUsingPage());

    mnemonic = new LOOPC(MNEMONIC + " 0x01");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE + 0x01, mnemonic.getByte());

    mnemonic = new LOOPC(MNEMONIC + " 0x0f");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE + 0x0f, mnemonic.getByte());

    mnemonic = new LOOPC(MNEMONIC + " :test");
    mnemonic.checkArgument();
    assertTrue(mnemonic.isLabel());

    Assertions.assertThrows(SyntaxError.class, () -> {
      LOOPC mnemonic1 = new LOOPC(MNEMONIC);
      mnemonic1.checkArgument();
    });

    Assertions.assertThrows(IllegalArgument.class, () -> {
      LOOPC mnemonic1 = new LOOPC(MNEMONIC + " 0x10");
      mnemonic1.checkArgument();
    });

  }

  @Test
  void testMnemonicFactory() throws SyntaxError {
    Mnemonic mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " 0x00", 0);
    assertEquals(LOOPC.class, mnemonic.getClass());

    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC, 0);
    });

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " 0x02", 0);
    assertEquals(BYTE_VALUE + 0x02, mnemonic.getByte());

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " 12", 0);
    assertEquals(BYTE_VALUE + 12, mnemonic.getByte());

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " 0b00001010", 0);
    assertEquals(BYTE_VALUE + 10, mnemonic.getByte());

  }
}
