package de.mcs.tools.sps.mnemonic;

import static org.junit.jupiter.api.Assertions.assertEquals;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import de.mcs.tools.sps.exceptions.IllegalArgument;
import de.mcs.tools.sps.exceptions.SyntaxError;

class TestPORT {

  private static final int PORT_BYTE_VALUE = 0x10;
  private final String MNEMONIC = "PORT";

  @Test
  void testMnemonic() throws SyntaxError {
    PORT mnemonic = new PORT(MNEMONIC + " #0x04");
    mnemonic.checkArgument();
    assertEquals(PORT_BYTE_VALUE + 0x04, mnemonic.getByte());

    mnemonic = new PORT(MNEMONIC + " #0x00");
    mnemonic.checkArgument();
    assertEquals(PORT_BYTE_VALUE + 0x00, mnemonic.getByte());

    mnemonic = new PORT(MNEMONIC + " #0x0f");
    mnemonic.checkArgument();
    assertEquals(PORT_BYTE_VALUE + 0x0f, mnemonic.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      PORT mno1 = new PORT(MNEMONIC + " 0x05");
      mno1.checkArgument();
    });

    Assertions.assertThrows(IllegalArgument.class, () -> {
      PORT mno1 = new PORT(MNEMONIC + " 0x10");
      mno1.checkArgument();
    });

    Assertions.assertThrows(IllegalArgument.class, () -> {
      PORT mno1 = new PORT(MNEMONIC + " #0x10");
      mno1.checkArgument();
    });
  }

  @Test
  void testMnemonicFactory() throws SyntaxError {
    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC, 0);
    });

    Mnemonic mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " #0x04", 0);
    assertEquals(PORT.class, mnemonic.getClass());

    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC + " 0x00", 0);
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC + " 5", 0);
    });

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " #3", 0);
    assertEquals(PORT_BYTE_VALUE + 3, mnemonic.getByte());

  }
}
